@@ -1,5 +1,4 @@
 import os
-import time
 from collections import defaultdict
 from typing import Mapping  # noqa
 from typing import Any, Dict, Optional, Tuple, cast
@@ -8,49 +7,74 @@ from _pytest.config import Config
 from _pytest.terminal import TerminalReporter
 from pytablewriter import TableWriterFactory
 from pytablewriter.style import Cell, Style
+from typepy import Bool, Integer, StrictLevel
+from typepy.error import TypeConversionError
 
-from ._const import BGColor, FGColor, Header, Ini
+from ._const import BGColor, Default, EnvVar, FGColor, Header, HelpMsg, Ini
 
 
 def pytest_addoption(parser):
     group = parser.getgroup("md report", "make test results report with markdown table format")
 
     group.addoption(
-        "--md-report", action="store_true", default=None, help="create markdown report",
+        "--md-report",
+        action="store_true",
+        default=None,
+        help=HelpMsg.MD_REPORT + HelpMsg.EXTRA_MSG_TEMPLATE.format(EnvVar.MD_REPORT),
     )
     group.addoption(
-        "--md-report-verbose", type=int, default=None, help="verbosity level",
+        "--md-report-verbose",
+        metavar="VERBOSITY_LEVEL",
+        type=int,
+        default=None,
+        help=HelpMsg.MD_REPORT_VERBOSE
+        + HelpMsg.EXTRA_MSG_TEMPLATE.format(EnvVar.MD_REPORT_VERBOSE),
     )
     group.addoption(
         "--md-report-color",
-        choices=["auto", "never"],
+        choices=["auto", "text", "never"],
         default=None,
-        help="""
-        auto: colorizing report for terminal with ANSI escape codes,
-        never: report without color"
-        """
+        help=HelpMsg.MD_REPORT_COLOR + HelpMsg.EXTRA_MSG_TEMPLATE.format(EnvVar.MD_REPORT_COLOR),
     )
 
     parser.addini(
-        Ini.MD_REPORT, type="bool", default=False, help="create markdown report",
+        Ini.MD_REPORT, type="bool", default=False, help=HelpMsg.MD_REPORT,
     )
     parser.addini(
-        Ini.MD_REPORT_VERBOSE, default=None, help="verbosity level for md report",
+        Ini.MD_REPORT_VERBOSE, default=None, help=HelpMsg.MD_REPORT_VERBOSE,
+    )
+    parser.addini(
+        Ini.MD_REPORT_COLOR, default=None, help=HelpMsg.MD_REPORT_COLOR,
     )
 
 
 def is_make_md_report(config: Config) -> bool:
     make_report = config.option.md_report
+
+    if make_report is None:
+        try:
+            make_report = Bool(
+                os.environ.get(EnvVar.MD_REPORT), strict_level=StrictLevel.MIN
+            ).convert()
+        except TypeConversionError:
+            make_report = None
+
     if make_report is None:
         make_report = config.getini(Ini.MD_REPORT)
-    if make_report is None:
-        make_report = os.environ.get("PYTEST_MD_REPORT")
+
     if make_report is None:
         return False
 
     return make_report
 
 
+def _to_int(value) -> Optional[int]:
+    try:
+        return Integer(value, strict_level=StrictLevel.MIN).convert()
+    except TypeConversionError:
+        return None
+
+
 def retrieve_verbosity_level(config: Config) -> int:
     verbosity_level = config.option.md_report_verbose
 
@@ -58,15 +82,10 @@ def retrieve_verbosity_level(config: Config) -> int:
         verbosity_level = None
 
     if verbosity_level is None:
-        verbosity_level = config.getini(Ini.MD_REPORT_VERBOSE)
-        if verbosity_level is not None:
-            try:
-                verbosity_level = int(verbosity_level)
-            except ValueError:
-                verbosity_level = None
+        verbosity_level = _to_int(os.environ.get(EnvVar.MD_REPORT_VERBOSE))
 
     if verbosity_level is None:
-        verbosity_level = os.environ.get("PYTEST_MD_REPORT_VERBOSE")
+        verbosity_level = _to_int(config.getini(Ini.MD_REPORT_VERBOSE))
 
     if verbosity_level is None:
         verbosity_level = config.option.verbose
@@ -76,10 +95,15 @@ def retrieve_verbosity_level(config: Config) -> int:
 
 def retrieve_report_color(config: Config) -> str:
     report_color = config.option.md_report_color
-    if report_color is None:
-        report_color = config.getini(Ini.MD_REPORT_VERBOSE)
-    if report_color is None:
-        report_color = os.environ.get("PYTEST_MD_REPORT_COLOR")
+
+    if not report_color:
+        report_color = os.environ.get(EnvVar.MD_REPORT_COLOR)
+
+    if not report_color:
+        report_color = config.getini(Ini.MD_REPORT_COLOR)
+
+    if not report_color:
+        return Default.COLOR
 
     return report_color
 
@@ -101,21 +125,28 @@ def _retrieve_stat_count_map(reporter: TerminalReporter) -> Dict[str, int]:
     return stat_count_map
 
 
-def retrieve_fg_bg_color(row: int, base_color: str, is_grayout):
-    bg_color = None  # type: Optional[str]
+class ColorRetriever:
+    def __init__(self, row: int, is_grayout: bool, report_color: str) -> None:
+        self.__row = row
+        self.__is_grayout = is_grayout
+        self.__report_color = report_color
 
-    if (row % 2) == 0:
-        fg_color = FGColor.GRAYOUT if is_grayout else base_color
-        bg_color = BGColor.EVEN_ROW
-    else:
-        fg_color = FGColor.GRAYOUT if is_grayout else base_color
-        bg_color = BGColor.ODD_ROW
+    def retrieve_fg_bg_color(self, base_color: str) -> Tuple[str, Optional[str]]:
+        bg_color = None  # type: Optional[str]
 
-    return (fg_color, bg_color)
+        if (self.__row % 2) == 0:
+            fg_color = FGColor.GRAYOUT if self.__is_grayout else base_color
+            bg_color = BGColor.EVEN_ROW if self.__report_color == "auto" else None
+        else:
+            fg_color = FGColor.GRAYOUT if self.__is_grayout else base_color
+            bg_color = BGColor.ODD_ROW if self.__report_color == "auto" else None
+
+        return (fg_color, bg_color)
 
 
 def style_filter(cell: Cell, **kwargs: Any) -> Optional[Style]:
     writer = kwargs["writer"]
+    report_color = kwargs["report_color"]
     fg_color = None
     bg_color = None
 
@@ -129,6 +160,8 @@ def style_filter(cell: Cell, **kwargs: Any) -> Optional[Style]:
 
         return Style(font_weight="bold")
 
+    retriever = ColorRetriever(cell.row, is_grayout, report_color)
+
     headers = writer.headers
     if headers[cell.col] in (Header.FILEPATH, Header.TESTFUNC):
         error_count = sum(
@@ -138,7 +171,7 @@ def style_filter(cell: Cell, **kwargs: Any) -> Optional[Style]:
             ]
         )
         if error_count > 0:
-            fg_color, bg_color = retrieve_fg_bg_color(cell.row, FGColor.ERROR, is_grayout)
+            fg_color, bg_color = retriever.retrieve_fg_bg_color(FGColor.ERROR)
             return Style(color=fg_color, bg_color=bg_color)
 
         skip_count = sum(
@@ -149,18 +182,18 @@ def style_filter(cell: Cell, **kwargs: Any) -> Optional[Style]:
             ]
         )
         if skip_count > 0:
-            fg_color, bg_color = retrieve_fg_bg_color(cell.row, FGColor.SKIP, is_grayout)
+            fg_color, bg_color = retriever.retrieve_fg_bg_color(FGColor.SKIP)
             return Style(color=fg_color, bg_color=bg_color)
 
-        fg_color, bg_color = retrieve_fg_bg_color(cell.row, FGColor.SUCCESS, is_grayout)
+        fg_color, bg_color = retriever.retrieve_fg_bg_color(FGColor.SUCCESS)
         return Style(color=fg_color, bg_color=bg_color)
 
     if headers[cell.col] in ("passed"):
-        fg_color, bg_color = retrieve_fg_bg_color(cell.row, FGColor.SUCCESS, is_grayout)
+        fg_color, bg_color = retriever.retrieve_fg_bg_color(FGColor.SUCCESS)
     elif headers[cell.col] in ("failed", "error"):
-        fg_color, bg_color = retrieve_fg_bg_color(cell.row, FGColor.ERROR, is_grayout)
+        fg_color, bg_color = retriever.retrieve_fg_bg_color(FGColor.ERROR)
     if headers[cell.col] in ("skipped", "xfailed", "xpassed"):
-        fg_color, bg_color = retrieve_fg_bg_color(cell.row, FGColor.SKIP, is_grayout)
+        fg_color, bg_color = retriever.retrieve_fg_bg_color(FGColor.SKIP)
 
     return Style(color=fg_color, bg_color=bg_color)
 
@@ -230,7 +263,9 @@ def make_md_report(
     writer.margin = 1
     writer.value_matrix = matrix
 
-    if retrieve_report_color(config) != "never":
+    report_color = retrieve_report_color(config)
+    if report_color != "never":
+        writer.style_filter_kwargs = {"report_color": report_color}
         writer.add_style_filter(style_filter)
         writer.add_col_separator_style_filter(col_separator_style_filter)
 
@@ -242,12 +277,5 @@ def pytest_unconfigure(config):
         return
 
     reporter = config.pluginmanager.get_plugin("terminalreporter")
-
-    try:
-        duration = time.time() - reporter._sessionstarttime
-    except AttributeError:
-        return
-
     stat_count_map = _retrieve_stat_count_map(reporter)
     reporter._tw.write(make_md_report(config, reporter, stat_count_map))
-    reporter._tw.write("\n")
